<?php
/**
 * Controller to handling updating widget
 *
 * @package wp-surveyer
 * @author InstaProduct
 * @link http://insta-product.com/
 */

if ( ! class_exists( 'IPSurv_Ajax_Update_widget' ))
{
    class IPSurv_Ajax_Update_widget
    {
        /**
         * Execution save options process
         *
         * @return void
         */
        public function exec()
        {
            $redirect = false;
            $is_add   = false;
            $error    = true;
            if ( ! empty( $_POST['isadd'] )) {
                $is_add = true;
            }
            if ( ! empty( $_POST['widget_id'] )) {
                $widgetId = absint( $_POST['widget_id'] );
                if ( $widget = get_post( $widgetId ) ) {
                    $this->_updateWidget( $widget );
                } else {
                    $widgetId = 0;
                }
            } else {
                $widgetId = $this->_addWidget();
            }
            if ( $widgetId )
            {
                $error = false;
                $model = new IPSurv_Model();
                $model->saveWidgetSetting( $widgetId );
            }
            if ( $error ) {
                $status   = 0;
                $widgetId  = 0;
                if ( $is_add ) {
                    $message = __( 'Failed create widget.', IPSURV_SLUG );
                } else {
                    $message = __( 'Failed save widget.', IPSURV_SLUG );
                }
            } else {
                $status = 1;
                if ( $is_add ) {
                    $message  = __( 'Successfully created widget.', IPSURV_SLUG );
                    $redirect = admin_url() . 'admin.php?page=wpsurv-widget&action=edit&id=' . $widgetId . '&create=1&tab=ipsurv-widget-section-content';
                } else {
                    $message = __( 'Successfully saved widget.', IPSURV_SLUG );
                }
            }
            $result = array(
                'status'     => $status,
                'message'    => $message,
                'widgetid'   => $widgetId
            );
            if ( $redirect ) {
                $result['redirect'] = $redirect;
            }
            echo json_encode( $result );
            die();
        }

        /**
         * Add new widget
         *
         * @return int
         */
        protected function _addWidget()
        {
            global $user_ID;

            if ( empty( $_POST['title'] )) {
                $title = 'Unknown';
            } else {
                $title = apply_filters( 'single_post_title', $_POST['title'] );
            }
            if ( $_POST['status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            $dataPost = array(
                'post_status'  => $status,
                'post_type'    => 'ipsurv-widget-item',
                'post_author'  => $user_ID,
                'post_title'   => $title,
                'post_content' => ''
            );
            $result = wp_insert_post( $dataPost );
            return $result;
        }

        /**
         * Update widget
         *
         * @param  object $widget
         * @return boolean
         */
        protected function _updateWidget( $widget )
        {
            $result   = 0;
            $dataPost = array();
            if ( ! empty( $_POST['title'] ) && $_POST['title'] != $widget->post_title ) {
                $dataPost['post_title'] = apply_filters( 'single_post_title', $_POST['title'] );
            }
            if ( $_POST['status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            if ( $status != $widget->post_status ) {
                $dataPost['post_status'] = $status;
            }
            if ( ! empty( $dataPost )) {
                $dataPost['ID'] = $widget->ID;
                $result = wp_update_post( $dataPost );
            }
            return $result;
        }
    }
}
