<?php
/**
 * Controller to handling updating survey
 *
 * @package wp-surveyer
 * @author InstaProduct
 * @link http://insta-product.com/
 */

if ( ! class_exists( 'IPSurv_Ajax_Update' ))
{
    class IPSurv_Ajax_Update
    {
        /**
         * Execution save options process
         *
         * @return void
         */
        public function exec()
        {
            $redirect = false;
            $is_add   = false;
            $error    = true;
            if ( ! empty( $_POST['isadd'] )) {
                $is_add = true;
            }
            if ( ! empty( $_POST['survey_id'] )) {
                $surveyId = absint( $_POST['survey_id'] );
                if ( $survey = get_post( $surveyId ) ) {
                    $this->_updateSurvey( $survey );
                } else {
                    $surveyId = 0;
                }
            } else {
                $surveyId = $this->_addSurvey();
            }
            if ( $surveyId )
            {
                $error = false;
                $model = new IPSurv_Model();
                $model->saveSetting( $surveyId );

                //Save Question
                $qmodel = new IPSurv_Model_Question();
                $qmodel->saveQuestion( $surveyId );
            }
            if ( $error ) {
                $status   = 0;
                $surveyId  = 0;
                if ( $is_add ) {
                    $message = __( 'Failed create survey.', IPSURV_SLUG );
                } else {
                    $message = __( 'Failed save survey.', IPSURV_SLUG );
                }
            } else {
                $status = 1;
                if ( $is_add ) {
                    $message  = __( 'Successfully created survey.', IPSURV_SLUG );
                    $redirect = admin_url() . 'admin.php?page=wpsurv&action=edit&id=' . $surveyId . '&create=1&tab=ipsurv-section-content';
                } else {
                    $message = __( 'Successfully saved survey.', IPSURV_SLUG );
                }
            }
            $result = array(
                'status'     => $status,
                'message'    => $message,
                'surveyid'   => $surveyId
            );
            if ( $redirect ) {
                $result['redirect'] = $redirect;
            }
            echo json_encode( $result );
            die();
        }

        /**
         * Add new survey
         *
         * @return int
         */
        protected function _addSurvey()
        {
            global $user_ID;

            if ( empty( $_POST['title'] )) {
                $title = 'Unknown';
            } else {
                $title = apply_filters( 'single_post_title', $_POST['title'] );
            }
            if ( ! empty( $_POST['slug'] )) {
                $slug  = apply_filters( 'pre_term_name', $_POST['slug'] );
            } else {
                $slug = sanitize_title( $_POST['title'] );
                $slug = wp_unique_post_slug( $slug, 0, 1, 'ipsurv-item', 0 );
            }
            if ( $_POST['status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            $dataPost = array(
                'post_status'  => $status,
                'post_type'    => 'ipsurv-item',
                'post_author'  => $user_ID,
                'post_title'   => $title,
                'post_name'    => $slug,
                'post_content' => ''
            );
            $result = wp_insert_post( $dataPost );
            return $result;
        }

        /**
         * Update survey
         *
         * @param  object $survey
         * @return boolean
         */
        protected function _updateSurvey( $survey )
        {
            $result   = 0;
            $dataPost = array();
            if ( ! empty( $_POST['title'] ) && $_POST['title'] != $survey->post_title ) {
                $dataPost['post_title'] = apply_filters( 'single_post_title', $_POST['title'] );
            }
            if ( ! empty( $_POST['slug'] ) && $_POST['slug'] != $survey->post_name ) {
                $dataPost['post_name'] = apply_filters( 'pre_term_name', $_POST['slug'] );
            }
            if ( $_POST['status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            if ( $status != $survey->post_status ) {
                $dataPost['post_status'] = $status;
            }
            if ( ! empty( $dataPost )) {
                $dataPost['ID'] = $survey->ID;
                $result = wp_update_post( $dataPost );
            }
            return $result;
        }
    }
}
