<?php
/**
 * Class model
 *
 * @package wp-surveyer
 * @author InstaProduct
 * @link http://insta-product.com/
 */

if ( ! class_exists( 'IPSurv_Model' )) {

    /**
     * Class for manage post setting
     */
    class IPSurv_Model
    {
        /**
         * Save setting survey
         *
         * @param  int $surveyId
         * @return void
         */
        public function saveSetting( $surveyId )
        {
            $surveyId = absint( $surveyId );
            $setting  = new IPSurv_Setting( $surveyId, '__ipsurv-setting' );
            $data     = $this->_getSettingData( $setting );
            $result   = $setting->setSettings( $data );
            return $result;
        }

        /**
         * Get setting data
         *
         * @return array
         */
        protected function _getSettingData( $config )
        {
            $setting = array();
            $data    = array_merge( $_POST, $_GET );

            //
            //
            // General
            //
            if ( isset( $data['status'] )) {
                $setting['status'] = apply_filters( 'pre_term_name', $data['status'] );
            }
            if ( isset( $data['restricted'] )) {
                $setting['restricted'] = apply_filters( 'pre_term_name', $data['restricted'] );
            }

            //
            //
            // Content
            //
            if ( isset( $data['start_button_label'] )) {
                $setting['start_button_label'] = apply_filters( 'single_post_title', $data['start_button_label'] );
            }
            if ( isset( $data['next_button_label'] )) {
                $setting['next_button_label'] = apply_filters( 'single_post_title', $data['next_button_label'] );
            }
            if ( isset( $data['error_message'] )) {
                $setting['error_message'] = apply_filters( 'single_post_title', $data['error_message'] );
            }
            if ( isset( $data['intro_title'] )) {
                $setting['intro_title'] = apply_filters( 'single_post_title', $data['intro_title'] );
            }
            if ( isset( $data['intro_subtitle'] )) {
                $setting['intro_subtitle'] = apply_filters( 'single_post_title', $data['intro_subtitle'] );
            }
            
            $introInfo = $config->getSetting( 'intro_info', '' );
            if ( ! empty( $data['intro_info'] ) && $introInfo != $data['intro_info'] ) {
                 $setting['intro_info'] = $data['intro_info'];
            } 
            elseif ( ! empty( $data['intro_info_editor'] ) 
                     && $introInfo != $data['intro_info_editor'] 
            ) {
                $setting['intro_info'] = $data['intro_info_editor'];
            } 
            
            $introTerm = $config->getSetting( 'intro_term', '' );
            if ( ! empty( $data['intro_term'] ) && $introTerm != $data['intro_term'] ) {
                 $setting['intro_term'] = $data['intro_term'];
            } 
            elseif ( ! empty( $data['intro_term_editor'] ) 
                     && $introTerm != $data['intro_term_editor'] 
            ) {
                $setting['intro_term'] = $data['intro_term_editor'];
            } 

            if ( isset( $data['finished_redirect'] )) {
                $setting['finished_redirect'] = apply_filters( 'pre_link_url', $data['finished_redirect'] );
            }
            if ( isset( $data['finished_title'] )) {
                $setting['finished_title'] = apply_filters( 'single_post_title', $data['finished_title'] );
            }
            
            
            $message = $config->getSetting( 'finished_message', '' );
            if ( ! empty( $data['finished_message'] ) && $message != $data['finished_message'] ) {
                 $setting['finished_message'] = $data['finished_message'];
            } 
            elseif ( ! empty( $data['finished_message_editor'] ) 
                     && $message != $data['finished_message_editor'] 
            ) {
                $setting['finished_message'] = $data['finished_message_editor'];
            } 

            //
            //
            // SEO
            //
            if ( isset( $data['seo_meta_title'] )) {
                $setting['seo_meta_title'] = apply_filters( 'single_post_title', $data['seo_meta_title'] );
            }
            if ( isset( $data['seo_meta_description'] )) {
                $setting['seo_meta_description'] = apply_filters( 'single_post_title', $data['seo_meta_description'] );
            }
            if ( isset( $data['seo_meta_keywords'] )) {
                $setting['seo_meta_keywords'] = apply_filters( 'single_post_title', $data['seo_meta_keywords'] );
            }
            if ( isset( $data['seo_graph_image'] )) {
                $setting['seo_graph_image'] = apply_filters( 'pre_link_url', $data['seo_graph_image'] );
            }
            if ( isset( $data['seo_favicon'] )) {
                $setting['seo_favicon'] = apply_filters( 'pre_link_url', $data['seo_favicon'] );
            }
            if ( isset( $data['seo_tracking_code'] )) {
                $setting['seo_tracking_code'] = $data['seo_tracking_code'];
            }

            //
            //
            //Design
            //
            if ( isset( $data['background_image'] )) {
                $setting['background_image'] = absint( $data['background_image'] );
            }
            if ( isset( $data['background_opacity'] )) {
                $setting['background_opacity'] = absint( $data['background_opacity'] );
            }
            if ( isset( $data['background_repeat'] )) {
                $setting['background_repeat'] = apply_filters( 'pre_term_name', $data['background_repeat'] );
            }

            if ( isset( $data['header_title'] )) {
                $setting['header_title'] = apply_filters( 'single_post_title', $data['header_title'] );
            }
            if ( isset( $data['header_logo'] )) {
                $setting['header_logo'] = absint( $data['header_logo'] );
            }
            if ( isset( $data['header_style'] )) {
                $setting['header_style'] = absint( $data['header_style'] );
            }
            if ( isset( $data['header_alignment'] )) {
                $setting['header_alignment'] = apply_filters( 'pre_term_name', $data['header_alignment'] );
            }
            if ( isset( $data['header_background_color'] )) {
                $setting['header_background_color'] = apply_filters( 'pre_term_name', $data['header_background_color'] );
            }
            if ( isset( $data['header_color'] )) {
                $setting['header_color'] = apply_filters( 'pre_term_name', $data['header_color'] );
            }
            if ( isset( $data['header_border_width'] )) {
                $setting['header_border_width'] = absint( $data['header_border_width'] );
            }
            if ( isset( $data['header_border_color'] )) {
                $setting['header_border_color'] = apply_filters( 'pre_term_name', $data['header_border_color'] );
            }
            if ( isset( $data['header_border_radius'] )) {
                $setting['header_border_radius'] = apply_filters( 'pre_term_name', $data['header_border_radius'] );
            }
            if ( isset( $data['header_style'] )) {
                $setting['header_style'] = absint( $data['header_style'] );
            }
            if ( isset( $data['header_padding_bottom'] )) {
                $setting['header_padding_bottom'] = absint( $data['header_padding_bottom'] );
            }
            if ( isset( $data['header_font_family'] )) {
                $setting['header_font_family'] = absint( $data['header_font_family'] );
            }
            if ( isset( $data['header_shadow'] )) {
                $setting['header_shadow'] = apply_filters( 'pre_term_name', $data['header_shadow'] );
            }

            if ( isset( $data['footer_text'] )) {
                $setting['footer_text'] = apply_filters( 'single_post_title', $data['footer_text'] );
            }
            if ( isset( $data['footer_style'] )) {
                $setting['footer_style'] = absint( $data['footer_style'] );
            }
            if ( isset( $data['footer_background_color'] )) {
                $setting['footer_background_color'] = apply_filters( 'pre_term_name', $data['footer_background_color'] );
            }
            if ( isset( $data['footer_color'] )) {
                $setting['footer_color'] = apply_filters( 'pre_term_name', $data['footer_color'] );
            }
            if ( isset( $data['footer_border_width'] )) {
                $setting['footer_border_width'] = absint( $data['footer_border_width'] );
            }
            if ( isset( $data['footer_border_color'] )) {
                $setting['footer_border_color'] = apply_filters( 'pre_term_name', $data['footer_border_color'] );
            }
            if ( isset( $data['footer_border_radius'] )) {
                $setting['footer_border_radius'] = apply_filters( 'pre_term_name', $data['footer_border_radius'] );
            }
            if ( isset( $data['footer_style'] )) {
                $setting['footer_style'] = absint( $data['footer_style'] );
            }
            if ( isset( $data['footer_padding_bottom'] )) {
                $setting['footer_padding_bottom'] = absint( $data['footer_padding_bottom'] );
            }
            if ( isset( $data['footer_font_family'] )) {
                $setting['footer_font_family'] = absint( $data['footer_font_family'] );
            }
            if ( isset( $data['footer_alignment'] )) {
                $setting['footer_alignment'] = apply_filters( 'pre_term_name', $data['footer_alignment'] );
            }


            if ( isset( $data['content_style'] )) {
                $setting['content_style'] = absint( $data['content_style'] );
            }
            if ( isset( $data['content_background_color'] )) {
                $setting['content_background_color'] = apply_filters( 'pre_term_name', $data['content_background_color'] );
            }
            if ( isset( $data['content_color'] )) {
                $setting['content_color'] = apply_filters( 'pre_term_name', $data['content_color'] );
            }
            if ( isset( $data['content_border_width'] )) {
                $setting['content_border_width'] = absint( $data['content_border_width'] );
            }
            if ( isset( $data['content_border_color'] )) {
                $setting['content_border_color'] = apply_filters( 'pre_term_name', $data['content_border_color'] );
            }
            if ( isset( $data['content_border_radius'] )) {
                $setting['content_border_radius'] = apply_filters( 'pre_term_name', $data['content_border_radius'] );
            }
            if ( isset( $data['content_style'] )) {
                $setting['content_style'] = absint( $data['content_style'] );
            }
            if ( isset( $data['content_padding_top'] )) {
                $setting['content_padding_top'] = absint( $data['content_padding_top'] );
            }
            if ( isset( $data['content_padding_bottom'] )) {
                $setting['content_padding_bottom'] = absint( $data['content_padding_bottom'] );
            }
            if ( isset( $data['content_margin_top'] )) {
                $setting['content_margin_top'] = absint( $data['content_margin_top'] );
            }
            if ( isset( $data['content_margin_bottom'] )) {
                $setting['content_margin_bottom'] = absint( $data['content_margin_bottom'] );
            }
            if ( isset( $data['content_font_family'] )) {
                $setting['content_font_family'] = absint( $data['content_font_family'] );
            }
            if ( isset( $data['content_button_style'] )) {
                $setting['content_button_style'] = apply_filters( 'pre_term_name', $data['content_button_style'] );
            }
            if ( isset( $data['content_button_style_color'] )) {
                $setting['content_button_style_color'] = apply_filters( 'pre_term_name', $data['content_button_style_color'] );
            }


            //
            //
            // Share
            //
            if ( isset( $data['share_status'] )) {
                $setting['share_status'] = apply_filters( 'pre_term_name', $data['share_status'] );
            }
            if ( isset( $data['share_facebook'] )) {
                $setting['share_facebook'] = apply_filters( 'pre_term_name', $data['share_facebook'] );
            }
            if ( isset( $data['share_twitter'] )) {
                $setting['share_twitter'] = apply_filters( 'pre_term_name', $data['share_twitter'] );
            }
            if ( isset( $data['share_google-plus'] )) {
                $setting['share_google-plus'] = apply_filters( 'pre_term_name', $data['share_google-plus'] );
            }
            if ( isset( $data['share_linkedin'] )) {
                $setting['share_linkedin'] = apply_filters( 'pre_term_name', $data['share_linkedin'] );
            }
            if ( isset( $data['share_tumblr'] )) {
                $setting['share_tumblr'] = apply_filters( 'pre_term_name', $data['share_tumblr'] );
            }
            if ( isset( $data['share_pinterest'] )) {
                $setting['share_pinterest'] = apply_filters( 'pre_term_name', $data['share_pinterest'] );
            }
            if ( isset( $data['share_stumbleupon'] )) {
                $setting['share_stumbleupon'] = apply_filters( 'pre_term_name', $data['share_stumbleupon'] );
            }
            if ( isset( $data['share_digg'] )) {
                $setting['share_digg'] = apply_filters( 'pre_term_name', $data['share_digg'] );
            }
            if ( isset( $data['share_button_style'] )) {
                $setting['share_button_style'] = apply_filters( 'pre_term_name', $data['share_button_style'] );
            }

            return $setting;
        }

        /**
         * Save setting widget
         *
         * @param  int $widgetId
         * @return void
         */
        public function saveWidgetSetting( $widgetId )
        {
            $widgetId = absint( $widgetId );
            $setting  = new IPSurv_Setting( $widgetId, '__ipsurv-widget-setting' );
            $data     = $this->_getWidgetSettingData( $setting );
            $result   = $setting->setSettings( $data );
            return $result;
        }

        /**
         * Get widget setting data
         *
         * @return array
         */
        protected function _getWidgetSettingData( $config )
        {
            $setting = array();
            $data    = array_merge( $_POST, $_GET );

            //
            //
            // General
            //
            if ( isset( $data['survey'] )) {
                $setting['survey'] = absint( $data['survey'] );
            }
            if ( isset( $data['status'] )) {
                $setting['status'] = apply_filters( 'pre_term_name', $data['status'] );
            }
            if ( isset( $data['restricted'] )) {
                $setting['restricted'] = apply_filters( 'pre_term_name', $data['restricted'] );
            }

            //
            //
            // Content
            //
            if ( isset( $data['widget_title'] )) {
                $setting['widget_title'] = apply_filters( 'single_post_title', $data['widget_title'] );
            }
            if ( isset( $data['widget_subtitle'] )) {
                $setting['widget_subtitle'] = apply_filters( 'single_post_title', $data['widget_subtitle'] );
            }
            if ( isset( $data['widget_logo'] )) {
                $setting['widget_logo'] = absint( $data['widget_logo'] );
            }
            if ( isset( $data['widget_deny_button'] )) {
                $setting['widget_deny_button'] = apply_filters( 'single_post_title', $data['widget_deny_button'] );
            }
            if ( isset( $data['widget_start_button'] )) {
                $setting['widget_start_button'] = apply_filters( 'single_post_title', $data['widget_start_button'] );
            }
            
            $widgetInfo = $config->getSetting( 'widget_info', '' );
            if ( ! empty( $data['widget_info'] ) && $widgetInfo != $data['widget_info'] ) {
                 $setting['widget_info'] = $data['widget_info'];
            } 
            elseif ( ! empty( $data['widget_info_editor'] ) 
                     && $widgetInfo != $data['widget_info_editor'] 
            ) {
                $setting['widget_info'] = $data['widget_info_editor'];
            }
            $widgetTerm = $config->getSetting( 'widget_term', '' );
            if ( ! empty( $data['widget_term'] ) && $widgetTerm != $data['widget_term'] ) {
                 $setting['widget_term'] = $data['widget_term'];
            } 
            elseif ( ! empty( $data['widget_term_editor'] ) 
                     && $widgetTerm != $data['widget_term_editor'] 
            ) {
                $setting['widget_term'] = $data['widget_term_editor'];
            } 

            //
            //
            //Design
            //
            if ( isset( $data['widget_style'] )) {
                $setting['widget_style'] = apply_filters( 'pre_term_name', $data['widget_style'] );
            }
            if ( isset( $data['widget_background_color'] )) {
                $setting['widget_background_color'] = apply_filters( 'pre_term_name', $data['widget_background_color'] );
            }
            if ( isset( $data['widget_color'] )) {
                $setting['widget_color'] = apply_filters( 'pre_term_name', $data['widget_color'] );
            }
            if ( isset( $data['widget_link_color'] )) {
                $setting['widget_link_color'] = apply_filters( 'pre_term_name', $data['widget_link_color'] );
            }
            if ( isset( $data['widget_border_width'] )) {
                $setting['widget_border_width'] = absint( $data['widget_border_width'] );
            }
            if ( isset( $data['widget_border_color'] )) {
                $setting['widget_border_color'] = apply_filters( 'pre_term_name', $data['widget_border_color'] );
            }
            if ( isset( $data['widget_border_radius'] )) {
                $setting['widget_border_radius'] = apply_filters( 'pre_term_name', $data['widget_border_radius'] );
            }
            if ( isset( $data['widget_padding'] )) {
                $setting['widget_padding'] = absint( $data['widget_padding'] );
            }
            if ( isset( $data['widget_font_family'] )) {
                $setting['widget_font_family'] = apply_filters( 'pre_term_name', $data['widget_font_family'] );
            }
            if ( isset( $data['widget_button_style'] )) {
                $setting['widget_button_style'] = apply_filters( 'pre_term_name', $data['widget_button_style'] );
            }
            if ( isset( $data['widget_button_style_color'] )) {
                $setting['widget_button_style_color'] = apply_filters( 'pre_term_name', $data['widget_button_style_color'] );
            }
            if ( isset( $data['widget_deny_button_style_color'] )) {
                $setting['widget_deny_button_style_color'] = apply_filters( 'pre_term_name', $data['widget_deny_button_style_color'] );
            }
            if ( isset( $data['widget_alignment'] )) {
                $setting['widget_alignment'] = apply_filters( 'pre_term_name', $data['widget_alignment'] );
            }

            //
            //
            //Effect
            //
            if ( isset( $data['widget_delay'] )) {
                $setting['widget_delay'] = absint( $data['widget_delay'] );
            }
            if ( isset( $data['showing_animation_type'] )) {
                $setting['showing_animation_type'] = apply_filters( 'pre_term_name', $data['showing_animation_type'] );
            }
            if ( isset( $data['showing_animation_speed'] )) {
                $setting['showing_animation_speed'] = apply_filters( 'pre_term_name', $data['showing_animation_speed'] );
            }
            if ( isset( $data['showing_audio'] )) {
                $setting['showing_audio'] = absint( $data['showing_audio'] );
            }
            if ( isset( $data['hiding_animation_type'] )) {
                $setting['hiding_animation_type'] = apply_filters( 'pre_term_name', $data['hiding_animation_type'] );
            }
            if ( isset( $data['hiding_animation_speed'] )) {
                $setting['hiding_animation_speed'] = apply_filters( 'pre_term_name', $data['hiding_animation_speed'] );
            }
            if ( isset( $data['hiding_audio'] )) {
                $setting['hiding_audio'] = absint( $data['hiding_audio'] );
            }
            return $setting;
        }

        /**
         * Get stats
         *
         * @return array
         */
        public function getStats()
        {
            global $wpdb;

            $table = $wpdb->prefix . 'ipsurv_stats';
            $sql   = "
                SELECT *
                FROM `$table`
            ";
            $results = $wpdb->get_row( $sql );
            return $results;
        }

        /**
         * Get stat
         *
         * @param  int $stat_id
         * @return array
         */
        public function getStat( $stat_id )
        {
            global $wpdb;

            $stat_id = absint( $stat_id );
            $table   = $wpdb->prefix . 'ipsurv_stats';
            $sql     = "
                SELECT *
                FROM `$table` WHERE stat_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $stat_id );
            $results = $wpdb->get_row( $query );
            return $results;
        }

        /**
         * Get stat value
         *
         * @param  int    $page_id
         * @param  string $type
         * @return array
         */
        public function getStatValue( $page_id, $type )
        {
            $count = 0;
            if ( $stat = $this->getStatBypage( $page_id )) {
                if ( isset( $stat->$type )) {
                    $count = (int)$stat->$type;
                }
            }
            return $count;
        }

        /**
         * Get stat by page
         *
         * @param  int $page_id
         * @return array
         */
        public function getStatBypage( $page_id )
        {
            global $wpdb;

            $page_id = absint( $page_id );
            $table   = $wpdb->prefix . 'ipsurv_stats';
            $sql     = "
                SELECT
                    SUM( `stat_view` ) AS `view`,
                    SUM( `stat_subscribe` ) AS `subscribe`,
                    SUM( `stat_popup` ) AS `popup`,
                    SUM( `stat_button` ) AS `button`
                FROM `$table` WHERE page_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $page_id );
            $results = $wpdb->get_row( $query );
            return $results;
        }

        /**
         * Get stat by page date
         *
         * @param  int    $page_id
         * @param  string $date
         * @return array
         */
        public function getStatBypageDate( $page_id, $date )
        {
            global $wpdb;

            $page_id = absint( $page_id );
            $table   = $wpdb->prefix . 'ipsurv_stats';
            $sql     = "
                SELECT *
                FROM `$table`
                WHERE page_id = $page_id and stat_date = '$date'
            ";
            $results = $wpdb->get_row( $sql );
            return $results;
        }

        /**
         * Get statistic data
         *
         * @return array
         */
        public function getStatisticData( $page_id, $type, $start = null, $end = null )
        {
            global $wpdb;

            $page_id = absint( $page_id );
            $table   = $wpdb->prefix . 'ipsurv_stats';
            $query   = "
                SELECT
                    DATE_FORMAT(stat_date, '%Y-%m-%d') AS gdate,";
            //Type value
            if ( $type == 'view' ) {
                $query .= 'SUM(stat_view) AS value';
            } elseif ( $type == 'subscribe' ) {
                $query .= 'SUM(stat_subscribe) AS value';
            } elseif ( $type == 'popup' ) {
                $query .= 'SUM(stat_popup) AS value';
            } elseif ( $type == 'button' ) {
                $query .= 'SUM(stat_button) AS value';
            }
            $query .= " FROM $table
                        WHERE page_id = $page_id";

            if ( ! empty( $start ) && ! empty( $end ))
            {
                if ( $start == $end ) {
                    $query .= " AND DATE_FORMAT( stat_date, '%Y-%m-%d' ) = '$start'";
                } else {
                    $query .= " AND stat_date BETWEEN '$start' AND '$end'";
                }
            }
            elseif ( ! empty( $start )) {
                $query .= " AND DATE_FORMAT( stat_date, '%Y-%m-%d' ) = '$start'";
            }
            elseif ( ! empty( $end )) {
                $query .= " AND DATE_FORMAT( stat_date, '%Y-%m-%d' ) = '$end'";
            }
            $query .= " GROUP BY gdate";
            $results = $wpdb->get_results( $query );
            return $results;
        }

        /**
         * Do save statistic
         *
         * @param object $page
         * @param string $type
         * @param int    $count
         * @return void
         */
        public function saveStat( $page, $type, $count = 1 )
        {
            $_types = array(
                'view',
                'subscribe',
                'popup',
                'button'
            );
            $key = 'stat_';
            if ( in_array( $type, $_types )) {
                $key .= $type;
            }
            $date = date( 'Y-m-d 00:00:00' );
            $data = array();
            if ( $stat = $this->getStatBypageDate( $page->ID, $date ))
            {
                $ccount = 0;
                if ( isset( $stat->$key )) {
                    $ccount = $stat->$key;
                }
                $data[$key] = $ccount + $count;
                $this->_updateStat( $stat->stat_id, $data );
            }
            else {
                $data['page_id']   = $page->ID;
                $data['stat_date'] = $date;
                $data[$key]        = $count;
                $this->_insertStat( $data );
            }
        }

        /**
         * Insert statistic data
         *
         * @param array $data
         * @return void
         */
        protected function _insertStat( $data )
        {
            global $wpdb;

            $table  = $wpdb->prefix . 'ipsurv_stats';
            $data   = (array)$data;

            $result = $wpdb->insert( $table, $data );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }

        /**
         * Update statistic data
         *
         * @param int   $id
         * @param array $data
         * @return void
         */
        protected function _updateStat( $id, $data )
        {
            global $wpdb;

            $table  = $wpdb->prefix . 'ipsurv_stats';
            $data   = (array)$data;
            $where  = array(
                'stat_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
    }
}
?>
