<?php
/**
 * Class model question
 *
 * @package wp-surveyer
 * @author InstaProduct
 * @link http://insta-product.com/
 */

if ( ! class_exists( 'IPSurv_Model_Question' )) {

    /**
     * Class for manage post setting
     */
    class IPSurv_Model_Question
    {
        /**
         * Get Questions
         *
         * @return array
         */
        public function getQuestions( $surveyId )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'ipsurv_question';
            $sql   = "
                SELECT * FROM $table WHERE post_id = %d ORDER BY qtn_order
            ";
            $surveyId  = absint( $surveyId );
            $query   = $wpdb->prepare( $sql, $surveyId );
            $results = $wpdb->get_results( $query );
            return $results;
        }

        /**
         * Get Question
         *
         * @param  int $id
         * @return array
         */
        public function getQuestion( $id )
        {
            global $wpdb;
            $id    = absint( $id );
            $table = $wpdb->prefix . 'ipsurv_question';
            $sql   = "
                SELECT * FROM $table WHERE qtn_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_row( $query );
            return $results;
        }

        /**
         * Get Answers
         *
         * @return array
         */
        public function getAnswers( $qtnId )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'ipsurv_answer';
            $sql   = "
                SELECT * FROM $table WHERE qtn_id = %d ORDER BY ans_order
            ";
            $qtnId   = absint( $qtnId );
            $query   = $wpdb->prepare( $sql, $qtnId );
            $results = $wpdb->get_results( $query );
            return $results;
        }

        /**
         * Get Answer
         *
         * @param  int $id
         * @return array
         */
        public function getAnswer( $id )
        {
            global $wpdb;
            $id    = absint( $id );
            $table = $wpdb->prefix . 'ipsurv_answer';
            $sql   = "
                SELECT * FROM $table WHERE ans_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_row( $query );
            return $results;
        }

        /**
         * Get Answer Order
         *
         * @param  int $qtnId
         * @return array
         */
        public function getAnswerOrder( $qtnId )
        {
            global $wpdb;
            $qtnId = absint( $qtnId );
            $table = $wpdb->prefix . 'ipsurv_answer';
            $sql   = "
                SELECT * FROM $table WHERE qtn_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $qtnId );
            $results = $wpdb->get_results( $query );
            $order   = 1;
            if ( ! empty( $results )) {
                $order = count( $results ) + 1;
            }
            return $order;
        }

        /**
         * Get Choices
         *
         * @param  int $surveyId
         * @return array
         */
        public function getChoices( $surveyId )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipsurv_choice';
            $query = "
                SELECT * FROM $table
                WHERE
                    survey_id = $surveyId
                ORDER BY cho_date
            ";
            $results = $wpdb->get_results( $query );
            return $results;
        }

        /**
         * Get Choice
         *
         * @param  int $id
         * @return array
         */
        public function getChoice( $id )
        {
            global $wpdb;
            $id    = absint( $id );
            $table = $wpdb->prefix . 'ipsurv_choice';
            $sql   = "
                SELECT * FROM $table WHERE cho_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_row( $query );
            return $results;
        }

        /**
         * Get Choice by survey
         *
         * @param  int $surveyId
         * @return array
         */
        public function getChoiceBySurvey( $surveyId )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'ipsurv_choice';
            $sql   = "
                SELECT * FROM $table WHERE survey_id = %d ORDER BY cho_date
            ";
            $surveyId = absint( $surveyId );
            $query    = $wpdb->prepare( $sql, $surveyId );
            $results  = $wpdb->get_results( $query );
            return $results;
        }

        /**
         * Get Choice by ip
         *
         * @param  int $surveyId
         * @param  string $ip
         * @return array
         */
        public function getChoiceByIp( $surveyId, $ip = '' )
        {
            global $wpdb;

            if ( empty( $ip )) {
                $ip = $_SERVER['REMOTE_ADDR'];
            }
            $table = $wpdb->prefix . 'ipsurv_choice';
            $sql   = "
                SELECT * FROM $table WHERE survey_id = %d AND cho_ip = %d ORDER BY cho_date
            ";
            $surveyId = absint( $surveyId );
            $query    = $wpdb->prepare( $sql, $surveyId, $ip );
            $results  = $wpdb->get_row( $query );
            return $results;
        }

        /**
         * Get Choice by user
         *
         * @param  int $userId
         * @return array
         */
        public function getChoiceByUser( $userId )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'ipsurv_choice';
            $sql   = "
                SELECT * FROM $table WHERE user_id = %d ORDER BY cho_date
            ";
            $userId   = absint( $userId );
            $query    = $wpdb->prepare( $sql, $userId );
            $results  = $wpdb->get_results( $query );
            return $results;
        }

        /**
         * Get Choice Answer
         *
         * @param  int $choiceId
         * @return array
         */
        public function getChoiceAnswer( $choiceId, $qtnId )
        {
            global $wpdb;
            $choiceId = absint( $choiceId );
            $qtnId    = absint( $qtnId );
            $table    = $wpdb->prefix . 'ipsurv_choice_answer';
            $sql      = "
                SELECT * FROM $table WHERE cho_id = %d AND qtn_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $choiceId, $qtnId );
            $results = $wpdb->get_row( $query );
            return $results;
        }

        /**
         * Get Choice Answer
         *
         * @param  int $qtnId
         * @param  array $answerIds
         * @return array
         */
        public function getChoiceAnswerCount( $qtnId, $answerIds )
        {
            global $wpdb;
            $answers  = '(' . implode( ',', $answerIds )  . ')';
            $qtnId    = absint( $qtnId );
            $table    = $wpdb->prefix . 'ipsurv_choice_answer';
            $query    = "
                SELECT COUNT(*) AS count
                FROM $table
                WHERE qtn_id = $qtnId
                AND choa_data IN $answers
            ";
            $results = $wpdb->get_var( $query );
            return $results;
        }

        /**
         * Get Choice Answer
         *
         * @param  int $qtnId
         * @param  array $answerIds
         * @return array
         */
        public function getAllChoiceAnswerCount( $qtnId )
        {
            global $wpdb;
            $qtnId    = absint( $qtnId );
            $table    = $wpdb->prefix . 'ipsurv_choice_answer';
            $query    = "
                SELECT COUNT(*) AS count
                FROM $table
                WHERE qtn_id = $qtnId
            ";
            $results = $wpdb->get_var( $query );
            return $results;
        }

        /**
         * Get Choice Answer
         *
         * @param  int $choiceId
         * @return array
         */
        public function getChoiceAnswers( $choiceId, $qtnId )
        {
            global $wpdb;
            $choiceId = absint( $choiceId );
            $qtnId    = absint( $qtnId );
            $table    = $wpdb->prefix . 'ipsurv_choice_answer';
            $sql      = "
                SELECT * FROM $table WHERE cho_id = %d AND qtn_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $choiceId, $qtnId );
            $results = $wpdb->get_results( $query );
            return $results;
        }

        /**
         * Is choice ?
         *
         * @param  int $surveyId
         * @return array
         */
        public function isChoice( $surveyId )
        {
            global $wpdb;
            $ip    = $_SERVER['REMOTE_ADDR'];
            $table = $wpdb->prefix . 'ipsurv_choice';
            $sql   = "
                SELECT * FROM $table WHERE survey_id = %d AND cho_ip = %d AND cho_status = 2 ORDER BY cho_date
            ";
            $surveyId = absint( $surveyId );
            $query    = $wpdb->prepare( $sql, $surveyId, $ip );
            $results  = $wpdb->get_results( $query );
            if ( empty( $results )) {
                return false;
            }
            return true;
        }

        /**
         * Get statistic data
         *
         * @return array
         */
        public function getStatisticData( $start = null, $end = null )
        {
            global $wpdb;

            $table = $wpdb->prefix . 'ipsurv_progress';
            $query = "
                SELECT
                    DATE_FORMAT(pro_update, '%Y-%m-%d') AS gdate,
                    COUNT(parc_id) AS total_parc,
                    COUNT(survey_id) AS total_survey
                FROM $table";

            if ( ! empty( $start ) && ! empty( $end ))
            {
                if ( $start == $end ) {
                    $query .= " WHERE DATE_FORMAT( pro_update, '%Y-%m-%d' ) = '$start'";
                } else {
                    $query .= " WHERE pro_update BETWEEN '$start' AND '$end'";
                }
            }
            elseif ( ! empty( $start )) {
                $query .= " WHERE DATE_FORMAT( pro_update, '%Y-%m-%d' ) = '$start'";
            }
            elseif ( ! empty( $end )) {
                $query .= " WHERE DATE_FORMAT( pro_update, '%Y-%m-%d' ) = '$end'";
            }
            $query .= " GROUP BY gdate";
            $results = $wpdb->get_results( $query );
            return $results;
        }

        /**
         * Insert Question
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insertQuestion( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipsurv_question';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }

        /**
         * Update Question
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function updateQuestion( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipsurv_question';
            $data   = (array)$data;
            $where  = array(
                'qtn_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }

        /**
         * Delete Question
         *
         * @param  int   $id
         * @return void
         */
        public function deleteQuestion( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipsurv_question';
            $where  = array(
                'qtn_id' => absint( $id )
            );

            $result = $wpdb->delete( $table, $where );
            return $result;
        }

        /**
         * Insert Answer
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insertAnswer( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipsurv_answer';
            $data   = (array)$data;

            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }

        /**
         * Update Answer
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function updateAnswer( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipsurv_answer';
            $data   = (array)$data;
            $where  = array(
                'ans_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }

        /**
         * Delete Answer
         *
         * @param  int   $id
         * @return void
         */
        public function deleteAnswer( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipsurv_answer';
            $where  = array(
                'ans_id' => absint( $id )
            );
            $result = $wpdb->delete( $table, $where );
            return $result;
        }

        /**
         * Delete Answers
         *
         * @param  int   $qtnId
         * @return void
         */
        public function deleteAnswers( $qtnId )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipsurv_answer';
            $where  = array(
                'qtn_id' => absint( $qtnId )
            );
            $result = $wpdb->delete( $table, $where );
            return $result;
        }

        /**
         * Insert Choice
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insertChoice( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipsurv_choice';
            $data   = (array)$data;

            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }

        /**
         * Update Choice
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function updateChoice( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipsurv_choice';
            $data   = (array)$data;
            $where  = array(
                'cho_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }

        /**
         * Delete Choice
         *
         * @param  int   $id
         * @return void
         */
        public function deleteChoice( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipsurv_choice';
            $where  = array(
                'cho_id' => absint( $id )
            );
            $result = $wpdb->delete( $table, $where );
            return $result;
        }

        /**
         * Insert Choice answer
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insertChoiceAnswer( $data, $format = array() )
        {
            global $wpdb;

            $table  = $wpdb->prefix . 'ipsurv_choice_answer';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }

        /**
         * Delete Choice answer
         *
         * @param  int   $choId
         * @return void
         */
        public function deleteChoiceAnswer( $choId )
        {
            global $wpdb;

            $table  = $wpdb->prefix . 'ipsurv_choice_answer';
            $where  = array(
                'cho_id' => absint( $choId )
            );
            $result = $wpdb->delete( $table, $where );
            return $result;
        }

        /**
         * Save question data
         *
         * @param  int $surveyId
         * @return void
         */
        public function saveQuestion( $surveyId )
        {
            $result   = true;
            $surveyId = absint( $surveyId );
            if ( empty( $_POST['question_data'] )) {
                return $result;
            }
            $questions = array();
            if ( $current = $this->getQuestions( $surveyId ) ){
                foreach( $current as $qtn ) {
                    $qtnid = $qtn->qtn_id;
                    $questions[$qtnid] = $qtn;
                }
            }
            $order = 1;
            $data  = json_decode( stripslashes($_POST['question_data'] ), true );
            foreach( $data as $item )
            {
                if ( empty( $item )) {
                    continue;
                }
                $itemId   = 0;
                $question = false;
                if ( ! empty( $item['id'] )) {
                    $itemId   = absint( $item['id'] );
                    $question = $this->getQuestion( $itemId );
                }
                if ( ! $question ) {
                    $itemId = $this->addQuestion( $surveyId, $item, $order );
                } else {
                    if ( array_key_exists( $itemId, $questions )) {
                        unset( $questions[$itemId] );
                    }
                    $this->changeQuestion( $itemId, $item, $order );
                }
                if ( ! empty( $item['answers'] )) {
                    $this->saveAnswer( $itemId, $item['answers'] );
                }
                $order++;
            }
            if ( ! empty( $questions )) {
                foreach( $questions as $id => $qtn )
                {
                    $this->deleteAnswers( $id );
                    $this->deleteQuestion( $id );
                }
            }
            return $result;
        }

        /**
         * Add question
         *
         * @param  int   $itemId
         * @param  array $data
         * @return void
         */
        public function addQuestion( $surveyId, $data, $order = 1 )
        {
            $content  = '';
            if ( ! empty( $data['question_content_editor'] )) {
                $content = $data['question_content_editor'];
            } elseif ( ! empty( $data['question_content'] )) {
                $content = $data['question_content'];
            }
            $type = 'text';
            if ( ! empty( $data['answer_type'] )) {
                $type = apply_filters( 'pre_term_name', $data['answer_type'] );
            }
            $status = 1;
            if ( ! empty( $data['question_status'] ) && $data['question_status'] == 'false' ) {
                $status = 0;
            }
            $toInsert = array(
                'post_id'       => $surveyId,
                'qtn_content'   => $content,
                'qtn_type'      => $type,
                'qtn_status'    => $status,
                'qtn_order'     => $order
            );
            $result = $this->insertQuestion( $toInsert );
            return $result;
        }

        /**
         * Change question
         *
         * @param  int   $itemId
         * @param  array $data
         * @return void
         */
        public function changeQuestion( $itemId, $data )
        {
            $q = $this->getQuestion( $itemId );
            
            if ( ! empty( $data['question_content'] ) && $q->qtn_content != $data['question_content'] ) {
                $toUpdate['qtn_content'] = $data['question_content'];
            } 
            elseif ( ! empty( $data['question_content_editor'] ) && $q->qtn_content != $data['question_content_editor'] ) {
                $toUpdate['qtn_content'] = $data['question_content_editor'];
            }
            
            if ( ! empty( $data['answer_type'] )) {
                $toUpdate['qtn_type'] = apply_filters( 'pre_term_name', $data['answer_type'] );
            }
            $status = 1;
            if ( ! empty( $data['question_status'] ) && $data['question_status'] == 'false' ) {
                $status = 0;
            }
            $toUpdate['qtn_status'] = $status;
            $result = $this->updateQuestion( $itemId, $toUpdate );
            return $result;
        }

        /**
         * Save answer data
         *
         * @param  int $questId
         * @return boolean
         */
        public function saveAnswer( $questId )
        {
            if ( empty( $_POST['answers'] ) || empty( $_POST['answer_ids'] )) {
                return false;
            }
            
            //
            //Delete answers
            $oldAnswers = $this->getAnswers( $questId );
            if ( ! empty( $oldAnswers ))
            {
                $oldIds = array();
                foreach( $oldAnswers as $oldAnswer ) {
                    $oldIds[] = $oldAnswer->ans_id;
                }
                $toDelete = array_diff( $oldIds, $_POST['answer_ids'] );
                if ( ! empty( $toDelete ))
                {
                    foreach( $toDelete as $delId ) {
                        $this->deleteAnswer( $delId );
                    }
                }
            }
            
            $answers = $_POST['answers'];
            $order   = 1;
            foreach( $_POST['answer_ids'] as $index => $id )
            {
                if ( ! isset( $answers[$index] )) {
                    break;
                }
                
                $answerId = absint( $id );
                if ( empty( $answerId ))
                {
                    $toInsert = array(
                        'qtn_id'        => $questId,
                        'ans_content'   => $answers[$index],
                        'ans_status'    => 1,
                        'ans_order'     => $order
                    );
                    $this->insertAnswer( $toInsert );
                }
                else 
                {
                    $toUpdate = array(
                        'ans_content'   => $answers[$index],
                        'ans_order'     => $order
                    );
                    $this->updateAnswer( $answerId, $toUpdate );
                }
                $order++;
            }
            
            return true;
        }

        /**
         * Add answer
         *
         * @param  int   $questId
         * @param  array $data
         * @param  int   $order
         * @return void
         */
        public function addAnswer( $questId, $data, $order = 1 )
        {
            $content  = '';
            if ( ! empty( $data['answer'] )) {
                $content = $data['answer'];
            }
            $toInsert = array(
                'qtn_id'        => $questId,
                'ans_content'   => $content,
                'ans_order'     => $order
            );
            $result = $this->insertAnswer( $toInsert );
            return $result;
        }

        /**
         * Change answer
         *
         * @param  int   $itemId
         * @param  array $data
         * @param  int   $order
         * @return void
         */
        public function changeAnswer( $itemId, $data, $order = 1 )
        {
            $toUpdate = array(
                'ans_order' => $order
            );
            if ( ! empty( $data['answer'] )) {
                $toUpdate['ans_content'] = $data['answer'];
            }
            $result = $this->updateAnswer( $itemId, $toUpdate );
            return $result;
        }
    }
}
