<?php
if ( ! class_exists( 'IPSurv_Viewer' ))
{
    /**
     * Viewer class
     *
     * @package wp-surveyer
     * @author InstaProduct
     * @link http://insta-product.com/
     */

    class IPSurv_Viewer
    {
        //{{ dashboard

        /**
         * Manage page viewer
         *
         * @return void
         */
        public function dashboard()
        {
            //Render view
            $this->view( 'admin/dashboard' );
        }

        //}}
        //{{ addnew

        /**
         * Add new page viewer
         *
         * @return void
         */
        public function addnew()
        {
            $this->manage();
        }

        //}}
        //{{ edit

        /**
         * Edit page viewer
         *
         * @return void
         */
        public function edit()
        {
            if ( empty( $_GET['id'] )) {
                wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPSURV_SLUG ) );
            }
            $id = absint( $_GET['id'] );
            if ( ! $survey = get_post( $id ) ) {
                wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPSURV_SLUG ) );
            }
            $this->manage( $survey );
        }

        //}}
        //{{ manage

        /**
         * Manage page viewer
         *
         * @param object $survey
         * @return void
         */
        public function manage( $survey = false )
        {
            //
            //Including script
            wp_enqueue_media();
            wp_enqueue_script('wp-color-picker');
            wp_enqueue_style('wp-color-picker');

            $surveyId = 0;
            if ( $survey ) {
                $surveyId = $survey->ID;
            }
            $qmodel      = new IPSurv_Model_Question();
            $setting     = new IPSurv_Setting( $surveyId, '__ipsurv-setting' );
            $form_action = admin_url( 'admin-ajax.php' ) . '?doajax=1&ipsurv=1&action=update';

            if ( $survey )
            {
                $pageTitle   = __( 'Edit Survey', IPSURV_SLUG );
                $submitLabel = __( 'Save Change', IPSURV_SLUG );
                $submitId    = 'ipsurv-edit-survey';
                $formId      = 'ipsurv-form-update';
                $submitIcon  = 'floppy-o';
                $pageId      = 'wpsurv';
                $this->permalink = $this->_getPermalink( $survey );

                $questions     = $qmodel->getQuestions( $survey->ID );
                $questionMaxId = 0;
                if ( ! empty( $questions ))
                {
                    $ids = array();
                    foreach( $questions as $qtn ){
                        $ids[] = $qtn->qtn_id;
                    }
                    $questionMaxId = max( $ids );
                }
                $choices = $qmodel->getChoices( $survey->ID );

            } else
            {
                $pageTitle     = __( 'Add Survey', IPSURV_SLUG );
                $submitLabel   = __( 'Add Survey', IPSURV_SLUG );
                $submitId      = 'ipsurv-add-survey';
                $formId        = 'ipsurv-form-add';
                $submitIcon    = 'plus-circle';
                $pageId        = 'wpsurv-addnew';
                $questions     = array();
                $choices       = array();
                $questionMaxId = 0;
            }
            $this->pageId        = $pageId;
            $this->pageTitle     = $pageTitle;
            $this->submitLabel   = $submitLabel;
            $this->submitId      = $submitId;
            $this->formId        = $formId;
            $this->submitIcon    = $submitIcon;
            $this->setting       = $setting;
            $this->survey        = $survey;
            $this->surveyId      = $surveyId;
            $this->questions     = $questions;
            $this->choices       = $choices;
            $this->questionMaxId = $questionMaxId;
            $this->qmodel        = $qmodel;

            $tab = 'ipsurv-section-general';
            if ( ! empty( $_REQUEST['tab'] )) {
                $tab = $_REQUEST['tab'];
            }
            $this->tab = $tab;

            //
            //Permalink root
            $home = get_home_url();
            $path = get_option( '__ipsurv_base_path', 'survey' );
            $root = $home . '/' . $path . '/';
            $this->permalink_root = $root;

            //Render view
            $this->view( 'admin/manage' );
        }

        //}}
        //{{ widget

        /**
         * Manage widget viewer
         *
         * @return void
         */
        public function widget()
        {
            //Render view
            $this->view( 'admin/widget' );
        }

        //}}
        //{{ widgetCreate

        /**
         * Add new page viewer
         *
         * @return void
         */
        public function widgetCreate()
        {
            $this->widgetManage();
        }

        //}}
        //{{ widgetEdit

        /**
         * Edit page viewer
         *
         * @return void
         */
        public function widgetEdit()
        {
            if ( empty( $_GET['id'] )) {
                wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPMTUBE_SLUG ) );
            }
            $id = absint( $_GET['id'] );
            if ( ! $widget = get_post( $id ) ) {
                wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPMTUBE_SLUG ) );
            }
            $this->widgetManage( $widget );
        }

        //}}
        //{{ widgetManage

        /**
         * Manage widget viewer
         *
         * @param object $widget
         * @return void
         */
        public function widgetManage( $widget = false )
        {
            //
            //Including script
            wp_enqueue_media();
            wp_enqueue_script('wp-color-picker');
            wp_enqueue_style('wp-color-picker');

            $widgetId = 0;
            if ( $widget ) {
                $widgetId = $widget->ID;
            }
            $setting     = new IPSurv_Setting( $widgetId, '__ipsurv-widget-setting' );
            $form_action = admin_url( 'admin-ajax.php' ) . '?doajax=1&ipsurv=1&action=update_widget';

            if ( $widget )
            {
                $pageTitle   = __( 'Edit Widget', IPSURV_SLUG );
                $submitLabel = __( 'Save Change', IPSURV_SLUG );
                $submitId    = 'ipsurv-edit-widget';
                $formId      = 'ipsurv-form-widget-update';
                $submitIcon  = 'floppy-o';
                $pageId      = 'wpsurv-widget';

            } else
            {
                $pageTitle   = __( 'Add Widget', IPSURV_SLUG );
                $submitLabel = __( 'Add Widget', IPSURV_SLUG );
                $submitId    = 'ipsurv-add-widget';
                $formId      = 'ipsurv-form-widget-add';
                $submitIcon  = 'plus-circle';
                $pageId      = 'wpsurv-widget-create';
            }
            $this->pageId      = $pageId;
            $this->pageTitle   = $pageTitle;
            $this->submitLabel = $submitLabel;
            $this->submitId    = $submitId;
            $this->formId      = $formId;
            $this->submitIcon  = $submitIcon;
            $this->setting     = $setting;
            $this->widget      = $widget;
            $this->widgetId    = $widgetId;

            $tab = 'ipsurv-widget-section-general';
            if ( ! empty( $_REQUEST['tab'] )) {
                $tab = $_REQUEST['tab'];
            }
            $this->tab = $tab;

            //Render view
            $this->view( 'admin/widget/manage' );
        }

        //}}
        //{{ setting

        /**
         * Setting page viewer
         *
         * @return void
         */
        public function setting()
        {
            $this->widgets = $this->_getWidgets();
            //Render view
            $this->view( 'admin/setting' );
        }

        //}}
        //{{ shortcodeBackend

        /**
         * Shortcode Backend viewer
         *
         * @return void
         */
        public function shortcodeBackend()
        {
            $this->widgets = $this->_getWidgets();
            //Render view
            $this->view( 'shortcode/backend' );
        }

        //}}
        //{{ shortcode

        /**
         * Shortcode frontend viewer
         *
         * @param array $atts
         * @return void
         */
        public function shortcode( $atts )
        {
              global $user_ID;

              if ( empty( $atts['widget'] )) {
                  return '';
              }
              //
              //Get content
              $widgetId = absint( $atts['widget'] );
              if ( ! $widget = get_post( $widgetId )) {
                  return '';
              }
              //
              //Check post status
              if ( $widget->post_status != 'publish' ) {
                  return '';
              }
              $this->widget    = $widget;
              $this->setting   = new IPSurv_Setting( $widgetId, '__ipsurv-widget-setting' );

              //Check restricted
              if ( empty( $user_ID ) && $this->setting->getSetting( 'restricted', 'true' ) == 'true' ) {
                  return '';
              }
              $this->args      = $atts;
              $this->permalink = $this->_getPermalink( $widget );

              //
              //Include view
              ob_start();
              $this->view( 'front/widget' );
              $output  = ob_get_contents();
              ob_end_clean();
              return $output;
        }

        //}}
        //{{ metabox

        /**
         * Metabox options viewer
         *
         * @return void
         */
        public function metabox( $post )
        {
            $this->post    = $post;
            $this->widgets = $this->_getWidgets();
            //Render view
            $this->view( 'admin/metabox' );
        }

        //}}
        //{{ widgetBackend

        /**
         * Widget Backend viewer
         *
         * @param array $instance
         * @param object $wp_widget
         * @return void
         */
        public function widgetBackend( $instance, $wp_widget )
        {
            $this->widgets   = $this->_getWidgets();
            $this->instance  = $instance;
            $this->wp_widget = $wp_widget;
            //Render view
            $this->view( 'widget/backend' );
        }

        //}}
        //{{ widgetFrontend

        /**
         * Widget frontend viewer
         *
         * @param array $atts
         * @param array $widgetArgs
         * @return void
         */
        public function widgetFrontend( $atts, $widgetArgs = array() )
        {
              global $user_ID;

              if ( empty( $atts['widget'] )) {
                  return '';
              }

              //
              //Get content
              $widgetId = absint( $atts['widget'] );
              if ( ! $widget = get_post( $widgetId )) {
                  return '';
              }
              //
              //Check post status
              if ( $widget->post_status != 'publish' ) {
                  return '';
              }
              $this->widgetArgs = $widgetArgs;
              $this->widget     = $widget;
              $this->setting    = new IPSurv_Setting( $widgetId, '__ipsurv-widget-setting' );
              //Check restricted
              if ( empty( $user_ID ) && $this->setting->getSetting( 'restricted', 'true' ) == 'true' ) {
                  return '';
              }
              $this->args       = $atts;
              $this->permalink  = $this->_getPermalink( $widget );

              //
              //Include view
              $this->view( 'front/widget/sidebar' );
        }

        //}}
        //{{ widgetPage

        /**
         * Widget Page
         *
         * @return void
         */
        public function widgetPage()
        {
              global $user_ID, $post;

              $object = $this->_widgetPageObject();
              if ( ! empty( $object['widget'] ) )
              {
                  $widget = $object['widget'];
                  $type   = $object['type'];

                  $this->widget  = $widget;
                  $this->setting = new IPSurv_Setting( $widget->ID, '__ipsurv-widget-setting' );

                  //Check restricted
                  $restricted = false;
                  if ( empty( $user_ID ) && $this->setting->getSetting( 'restricted', 'true' ) == 'true' ) {
                      $restricted = true;
                  }

                  if ( $widget->post_status == 'publish' && ! $restricted )
                  {
                      $position = 'bottom_right';
                      $overlay  = 'false';
                      if ( $type == 'page' ) {
                          $position = get_post_meta( $post->ID, '__ipsurv_page_widget_position', true );
                          $overlay  = get_post_meta( $post->ID, '__ipsurv_page_widget_overlay', true );
                      } elseif ( $type == 'general' ) {
                          $position = get_option( '__ipsurv_general_widget_position' );
                          $overlay  = get_option( '__ipsurv_general_widget_overlay' );
                      } elseif ( $type == 'home' ) {
                          $position = get_option( '__ipsurv_home_widget_position' );
                          $overlay  = get_option( '__ipsurv_home_widget_overlay' );
                      }
                      if ( empty( $position )) {
                          $position = 'bottom_right';
                      }
                      if ( empty( $overlay )) {
                          $overlay  = 'false';
                      }
                      $this->position  = $position;
                      $this->overlay   = $overlay;
                      $this->type      = $type;

                      //
                      //Get survey
                      $permalink = '#';
                      if ( $surveyId = $this->setting->getSetting( 'survey' )) {
                          if ( $survey = get_post( $surveyId )) {
                              $permalink = $this->_getPermalink( $survey );
                          }
                      }
                      $this->permalink = $permalink;

                      //
                      //Include view
                      $this->view( 'front/widget/page' );
                  }
              }
        }

        //}}
        //{{ _widgetPageObject

        /**
         * Get widget Page object
         *
         * @return void
         */
        protected function _widgetPageObject()
        {
            global $post;

            $widgetId = 0;
            $widget   = false;
            $type     = 'general';
            $general  = get_option( '__ipsurv_general_widget' );
            if ( ! empty( $general )) {
                $widgetId = absint( $general );
                $type     = 'general';
            } else
            {
                if ( ( is_page() || is_single() ) && ! empty( $post )) {
                    $widgetId = get_post_meta( $post->ID, '__ipsurv_page_widget', true );
                    $type     = 'page';
                } elseif ( is_home() ) {
                    $widgetId = get_option( '__ipsurv_home_widget' );
                    $type     = 'home';
                }
            }
            if ( ! empty( $widgetId )) {
                $widget = get_post( $widgetId );
            }
            return array(
                'widget' => $widget,
                'type'   => $type
            );
        }

        //}}
        //{{ surveyPage

        /**
         * Widget frontend viewer
         *
         * @return void
         */
        public function surveyPage()
        {
            global $wp_query, $user_ID;
            if ( ( isset( $wp_query->query['ipsurv'] ) && isset( $wp_query->query['survey_name'] ) ) ||
                 isset( $wp_query->query['survey_id'] ) )
            {
                $page = $wp_query->post;

                if ( $page->post_status == 'publish' )
                {
                    $setting = new IPSurv_Setting( $page->ID, '__ipsurv-setting' );

                    //Check restricted
                    if ( empty( $user_ID ) && $setting->getSetting( 'restricted', 'true' ) == 'true' )
                    {
                        $redirect  = $this->permalink;
                        $login_url = wp_login_url( $redirect );
                        wp_redirect( $login_url );
                        exit;
                    }

                    $this->query     = $wp_query;
                    $this->setting   = $setting;
                    $this->page      = $page;
                    $this->cssurl    = IPSURV_BASE_URL . 'assets/css/';
                    $this->jsurl     = IPSURV_BASE_URL . 'assets/js/';
                    $this->imgurl    = IPSURV_BASE_URL . 'assets/img/';
                    $this->permalink = $this->_getPermalink( $page );

                    $qmodel          = new IPSurv_Model_Question();
                    $this->questions = $qmodel->getQuestions( $page->ID );
                    $this->isChoice  = $qmodel->isChoice( $page->ID );
                    $this->qmodel    = $qmodel;

                    $title = $this->setting->getSetting( 'seo_meta_title' );
                    $title = trim( $title );
                    if ( empty( $title )) {
                        $title = $this->page->post_title;
                    }
                    $this->pageTitle = stripslashes( $title );

                    //Render view
                    $this->view( 'front/survey' );
                    exit;
                }
                else
                {
                    $query->query_vars['post_type'] = 'ipsurv-item-nothing';
                }
            }
        }

        //}}
        //{{ surveyResult

        /**
         * Get survey result
         *
         * @param object $survey
         * @param object $quetion
         * @return void
         */
        public function surveyResult( $survey, $question )
        {
              $this->model    = new IPSurv_Model_Question();
              $this->survey   = $survey;
              $this->question = $question;
              $this->choices  = $this->model->getChoices( $survey->ID );

              //
              //Include view
              ob_start();
              $this->view( 'admin/manage/result/' . $question->qtn_type );
              $output  = ob_get_contents();
              ob_end_clean();
              return $output;
        }

        //}}
        //{{ getGoogleFont

        /**
         * Get Google font
         *
         * @return string
         */
        public function getGoogleFont()
        {
            $fonts   = array();
            $fonts[] = $this->setting->getSetting( 'header_font_family', 'roboto' );
            $fonts[] = $this->setting->getSetting( 'footer_font_family', 'roboto' );
            $fonts[] = $this->setting->getSetting( 'content_font_family', 'roboto' );
            $fonts   = array_unique( $fonts );

            $fams  = array();
            $links = IPSurv_Font::getLinks();
            foreach( $fonts as $font )
            {
                if ( array_key_exists( $font, $links ))
                {
                    $fams[] = $links[$font];
                }
            }

            $family  = implode( '|', $fams );
            $url     = 'http://fonts.googleapis.com/css?family=' . $family;
            return $url;
        }

        //}}
        //{{ view

        /**
         * Get admin view file
         *
         * @param string $filename
         * @return void
         */
        public function view( $filename )
        {
            $path  = IPSURV_VIEW;
            $path .= strtolower( $filename ) . '.php';
            if ( file_exists( $path )) {
                include( $path );
            }
        }

        //}}
        //{{ _getPermalink

        /**
         * Get link view
         *
         * @param  object $item
         * @return string
         */
        protected function _getPermalink( $item )
        {
            $home = get_home_url();
            $base = get_option( '__ipsurv_base_path', 'survey' );

            if ( get_option( 'permalink_structure' ) )  {
                $link = $home . "/$base/" . $item->post_name;
            } else {
                $link = $home . '/?survey_id=' . $item->ID . '&ipsurv=1';
            }

            return $link;
        }

        //}}
        //{{ _getWidgets

        /**
         * Get widgets list
         *
         * @return array
         */
        protected function _getWidgets()
        {
            $args = array(
                'orderby'        => 'post_title',
                'order'          => 'DESC',
                'post_type'      => 'ipsurv-widget-item',
                'post_status'    => array('publish'),
                'posts_per_page' => -1,
                'paged'          => 1
            );
            $query   = new WP_Query;
            $widgets = $query->query( $args );

            return $widgets;
        }

        //}}
        //{{ getShareLink

        /**
         * Get share link
         *
         * @param  string $type
         * @return string
         */
        public function getShareLink( $type )
        {
            $pagetitle = urlencode( $this->pageTitle );
            $pageurl   = urlencode( $this->permalink );
            $actions   =  array(
                'facebook'      => 'http://www.facebook.com/share.php?u=' . $pageurl,
                'twitter'       => 'https://twitter.com/share?url=' . $pageurl . '&text=' . $pagetitle,
                'google-plus'   => 'https://plusone.google.com/_/+1/confirm?hl=en&url=' . $pageurl,
                'pinterest'     => 'http://pinterest.com/pin/create/button/?url=' . $pageurl . '&media=&description=' . $pagetitle,
                'linkedin'      => 'http://www.linkedin.com/shareArticle?mini=true&url=' . $pageurl .'&title=' . $pagetitle,
                'tumblr'        => 'http://www.tumblr.com/share?v=3&u=' . $pageurl,
                'stumbleupon'   => 'http://www.stumbleupon.com/submit?url=' . $pageurl . '&title=' . $pagetitle,
                'digg'          => 'http://digg.com/submit?url=' . $pageurl . '&title=' . $pagetitle,
                'email'         => 'mailto:?subject=' . $pagetitle
            );
            $link = '#';
            if ( array_key_exists( $type, $actions )) {
                $link = $actions[$type];
            }
            return $link;
        }

        //}}
    }
}
